VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsGradients"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'\ CLASS GRADIENTS
'\ SIMON BRIDGE, 2005 mailto:simon.bridge@three.com.au

'\ A HANDY WAY OF MAKING BORING VB6 FORMS LOOK INTERESTING AND CLEVER
'\ GRADIENTS ARE ASSIGNED TO SQUARE SHAPE OBJECTS



Private Type TRIVERTEX
    x As Long
    y As Long
    Red As Integer 'Ushort value
    Green As Integer 'Ushort value
    Blue As Integer 'ushort value
    Alpha As Integer 'ushort
End Type

Private Type GRADIENT_RECT
    UpperLeft As Long  'In reality this is a UNSIGNED Long
    LowerRight As Long 'In reality this is a UNSIGNED Long
End Type
Const GRADIENT_FILL_RECT_H As Long = &H0 'In this mode, two endpoints describe a rectangle. The rectangle is
'defined to have a constant color (specified by the TRIVERTEX structure) for the left and right edges. GDI interpolates
'the color from the top to bottom edge and fills the interior.
Const GRADIENT_FILL_RECT_V  As Long = &H1 'In this mode, two endpoints describe a rectangle. The rectangle
' is defined to have a constant color (specified by the TRIVERTEX structure) for the top and bottom edges. GDI interpolates
' the color from the top to bottom edge and fills the interior.
Const GRADIENT_FILL_TRIANGLE As Long = &H2 'In this mode, an array of TRIVERTEX structures is passed to GDI
'along with a list of array indexes that describe separate triangles. GDI performs linear interpolation between triangle vertices
'and fills the interior. Drawing is done directly in 24- and 32-bpp modes. Dithering is performed in 16-, 8.4-, and 1-bpp mode.
Const GRADIENT_FILL_OP_FLAG As Long = &HFF


Private WithEvents FRM      As Form
Attribute FRM.VB_VarHelpID = -1
Private blDraw              As Boolean
Private Gradients           As New Collection
'\ GDI Gradient Function
Private Declare Function GdiGradientFillRect Lib "gdi32" Alias "GdiGradientFill" (ByVal hdc As Long, pVertex As TRIVERTEX, ByVal dwNumVertex As Long, pMesh As GRADIENT_RECT, ByVal dwNumMesh As Long, ByVal dwMode As Long) As Long

Private Function LongToUShort(Unsigned As Long) As Integer
    'A small function to convert from long to unsigned short
    LongToUShort = CInt(Unsigned - &H10000)
End Function

Public Sub GetFormRef(inFRM As Form)
'\ Retrieve a reference to the form to paint on
Set FRM = inFRM
    FRM.ScaleMode = vbTwips '\ set the scale mode to twips for consistency
    
End Sub

Public Sub AddShape(aSHP As Shape, iColourFrom As Long, iColourTo As Long, GradientType As GradientTypeConstants)
'\ Append a new shape ( gradient )

Dim NewGradient As New clsGradient  '\ clsGradient only contains a few public variables
                                    '\ Would have used a User Defined Type, but UDT's cannot be used in public
                                    '\ Members of Class Objects.
                                    '\ Damned Microsoft.
    
    With NewGradient
        Set .SHP = aSHP
        .SHP.Visible = False
        .iColourFrom = iColourFrom
        .iColourTo = iColourTo
        .GradientType = GradientType
    End With
    
Gradients.Add NewGradient, "G" & Gradients.Count + 1    '\ Append to the Gradients Collection

blDraw = True   '\ Dont attempt to draw anything until a shape has been set

End Sub

Private Sub LongRGB(iColour As Long, ByRef R As Integer, ByRef G As Integer, ByRef b As Integer)
'\ TIS NOT USED BUT TIS A USEFULL ROUTINE

Dim iR As Long
Dim iG As Long
Dim iB As Long

    iR = iColour Mod &H100  'get red component
    iColour = iColour \ &H100  'divide
    iG = iColour Mod &H100 'get green component
    iColour = iColour \ &H100  'divide
    iB = iColour Mod &H100
    
    R = iR
    G = iG
    b = iB
    
End Sub
Private Sub setTriVertexColor(tTV As TRIVERTEX, lColor As Long)
'\ This is a routine to apply standard RGB colour values to the god-awful bloody  Trivertex Structure.

Dim lRed As Long
Dim lGreen As Long
Dim lBlue As Long

    '\ Seperate The Colour Components
    lRed = (lColor And &HFF&) * &H100&
    lGreen = (lColor And &HFF00&)
    lBlue = (lColor And &HFF0000) \ &H100&
    
    '\ assign them to the trivertex structure
    setTriVertexColorComponent tTV.Red, lRed
    setTriVertexColorComponent tTV.Green, lGreen
    setTriVertexColorComponent tTV.Blue, lBlue
   
End Sub

Private Sub setTriVertexColorComponent(ByRef iColor As Integer, ByVal lComponent As Long)
'\ Used by setTriVertexColor

   If (lComponent And &H8000&) = &H8000& Then
      iColor = (lComponent And &H7F00&)
      iColor = iColor Or &H8000
   Else
      iColor = lComponent
   End If
   
End Sub

Private Sub FRM_Paint()
'\ This is the Paint Event of the referenced form

    Dim GRAD        As clsGradient      '\ This contains the info to get the two points and the colours
    Dim vert(1)     As TRIVERTEX        '\ We are going to define two points ( top-left, bottom-right )
    Dim gRect       As GRADIENT_RECT    '\ Need this to tell the GDI which element of the Vert array is the top left, bottom right etc
    
    If blDraw Then
        For Each GRAD In Gradients  '\ Enumerate the Gradients Collection
            '\ Top Left
            gRect.UpperLeft = 0
            With vert(0)
                '\ Use the Shape's TOP and LEFT properties
                .x = GRAD.SHP.Left / Screen.TwipsPerPixelX  '\ Convert Logical Coordinates to Pixels
                .y = GRAD.SHP.Top / Screen.TwipsPerPixelY
                '\ Put the colour information into the structure
                setTriVertexColor vert(0), GRAD.iColourFrom
            End With
    
            '\ Lower Right
            gRect.LowerRight = 1
            With vert(1)
                '\ Calculate the Shapes Bottom and Right Properties
                .x = (GRAD.SHP.Left + GRAD.SHP.Width) / Screen.TwipsPerPixelX
                .y = (GRAD.SHP.Top + GRAD.SHP.Height) / Screen.TwipsPerPixelX
                '\ Put the colour information into the structure
                setTriVertexColor vert(1), GRAD.iColourTo
            End With
            '\ Paint the gradient onto the form's Device Context (hDC)
            '\ The first element of the Vert() array is passed to the GDI, along with how many elements
            GdiGradientFillRect FRM.hdc, vert(0), 2, gRect, 1, GRAD.GradientType
            
        Next GRAD
    End If
End Sub
